/**
 * Test where the value is of any type.
 * 
 * @param value {any} - The value to test.
 */
const is_any = (value) => {
  return !!value;
};

/**
 * Test where the value is of array type.
 * 
 * @param value {array} - The value to test.
 */
const is_array = (value) => {
  return !!Array.isArray(value);
};

/**
 * Test where the value is of boolean type.
 * 
 * @param value {boolean} - The value to test.
 */
const is_boolean = (value) => {
  return (value === true || value === false);
};

/**
 * Test where the value is of float type.
 * 
 * @param value {float} - The value to test.
 */
const is_float = (value) => {
  return Number(value) === value && value % 1 !== 0;
};

/**
 * Test where the value is of function type.
 * 
 * @param value {function} - The value to test.
 */
const is_function = (value) => {
  return typeof value === 'function';
};

/**
 * Test where the value is of integer type.
 * 
 * @param value {integer} - The value to test.
 */
const is_integer = (value) => {
  return Number(value) === value && value % 1 === 0;
};

/**
 * Test where the value is of null type.
 * 
 * @param value {null} - The value to test.
 */
const is_null = (value) => {
  return value === null;
};

/**
 * Test where the value is of number type.
 * 
 * @param value {number} - The value to test.
 */
const is_number = (value) => {
  return Number(value) === value;
};

/**
 * Test where the value is of object type.
 * 
 * @param value {object} - The value to test.
 */
const is_object = (value) => {
  return !!(value && typeof value === "object" && !Array.isArray(value));
};

/**
 * Test where the value is of string type.
 * 
 * @param value {string} - The value to test.
 */
const is_string = (value) => {
  return typeof value === "string";
};

/**
 * Test where the value is of undefined type.
 * 
 * @param value {undefined} - The value to test.
 */
const is_undefined = (value) => {
  return typeof value === 'undefined';
};

export default {
	is_any,
	is_array,
	is_boolean,
	is_float,
	is_function,
	is_integer,
	is_null,
	is_number,
	is_object,
	is_string,
	is_undefined,
};
